#if !defined HATP_TOOLS_HH_
#define HATP_TOOLS_HH_

#include<iostream>
#include<sstream>
#include<typeinfo>
#include<vector>
#include<list>
#include<set>

#include "LightEntity.hh"
#include "Entity.hh"

namespace HATP_TOOLS {

/**
 * return true if vec1 contents is the same as vec2 contents
 */
template<typename T> bool operator==(std::vector<T> const& vec1, std::vector<T> const& vec2) {
	typename std::list<T> l1,l2;
	unsigned int i;
	for(i=0 ; i<vec1.size() ; ++i) l1.push_back(vec1[i]);
	for(i=0 ; i<vec2.size() ; ++i) l2.push_back(vec2[i]);
	l1.sort();
	l2.sort();
	return l1==l2;
}

/**
 * return true if val is in vec
 */
template<typename T> bool operator>>(T val, std::vector<T> const& vec) {
	for(unsigned int i=0 ; i<vec.size() ; ++i)
		if(vec[i]==val) return true;
	return false;
}

/**
 * try a lexical_cast on val to change it in T2 and search
 * resulting value in Set
 */
template<typename T> bool operator>>(T val, std::set<T> const& Set) {
	return Set.find(val) != Set.end();
}

/**
 * return true if val is not in vec (add by jguitton 11/05/10)
 */
template<typename T> bool operator^=(T val, std::vector<T> const& vec) {
	for(unsigned int i=0 ; i<vec.size() ; ++i)
		if(vec[i]==val) return false;
	return true;
}

/**
 * try a lexical_cast on val to change it in T2 and search
 * resulting value in Set. return true if val is NOT in Set
 */
template<typename T> bool operator^=(T val, std::set<T> const& Set) {
	return Set.find(val) == Set.end();
}

bool operator>>(Entity const* val, std::set<Entity*> const& Set);
bool operator>>(LightEntity const* val, std::set<LightEntity*> const& Set);
bool operator^=(Entity const* val, std::set<Entity*> const& Set);
bool operator^=(LightEntity const* val, std::set<LightEntity*> const& Set);

/**
 * return true if all members of subVec are in vec
 */
template<typename T> bool isSubVector(std::vector<T> const& subVec, std::vector<T> const& vec) {
	for(unsigned int i=0 ; i<subVec.size() ; ++i)
		if( !(subVec[i] >> vec) ) return false;
	return true;
}

/**
 * return contents of vec in a string (this is done by an ostringstream)
 */
template<typename T> std::string vectorToString(std::vector<T> const& vec) {
	std::ostringstream st;
	st << "{";
	for(unsigned int i=0 ; i<vec.size() ; ++i){
		st << vec[i];
		if(i < vec.size()-1) st << ", ";
	}
	st << "}";
	return st.str();
}

/**
 * merge vec1 and vec2 contents and return it
 * (members in vec1 and vec2 appears once)
 */
template<typename T> std::vector<T> vectorMerging(std::vector<T> const& vec1, std::vector<T> const& vec2) {
	std::vector<T> result;
	unsigned int i;
	for(i=0 ; i<vec1.size() ; ++i)
		if( !(vec1[i] >> result) ) result.push_back(vec1[i]);

	for(i=0 ; i<vec2.size() ; ++i)
		if( !(vec2[i] >> result) ) result.push_back(vec2[i]);
	return result;
}

/**
 * insert result in Set
 */
template<typename T> void operator<<=(std::set<T>& Set, T val) {
	Set.insert(val);
}

/**
 * extract result from Set
 */
template<typename T> void operator>>=(std::set<T>& Set, T val) {
	typename std::set<T>::iterator it;
	it = Set.find(val);
	if(it != Set.end()) Set.erase(it);
}

} // end namespace HATP_TOOLS

#endif
